'use strict';

exports.__esModule = true;
exports['default'] = stringify;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _isRegexp = require('is-regexp');

var _isRegexp2 = _interopRequireDefault(_isRegexp);

var _isobject = require('isobject');

var _isobject2 = _interopRequireDefault(_isobject);

function handlePrimitive(x) {
  var t = typeof x;
  if (x === null || x === undefined || t === 'number' || t === 'boolean' || _isRegexp2['default'](x)) {
    return String(x);
  }
  return null;
}

function handleFunction(x) {
  if (typeof x === 'function') {
    return String(x).replace(/{[\s\S]*}/, '=> ...').replace(/.*\(/, '(');
  }
  return null;
}

function handleString(x) {
  if (typeof x === 'string') {
    return '"' + x + '"';
  }
  return null;
}

function handleDate(x) {
  if (x instanceof Date) {
    return 'Date(\'' + x.toISOString() + '\')';
  }
  return null;
}

function handleArray(x, next, seen) {
  if (Array.isArray(x)) {
    if (x.length === 0) {
      return '[]';
    }
    seen.push(x);
    var result = '[' + x.map(next).join(', ') + ']';
    seen.pop();
    return result;
  }
  return null;
}

function wrapJsxProp(x) {
  return x[0] === '"' ? x : '{' + x + '}';
}

function wrapJsxChildren(x) {
  if (x[0] === '<') {
    return x;
  }
  if (x[0] === '"') {
    return x.slice(1, x.length - 1);
  }
  return '{' + x + '}';
}

function handleJsx(x, next, seen, indent) {
  // we'd better use TestUtils.isReactElement(),
  // but hopefully we'll avoid import whole `React/addons` with this tiny hack
  if (x && x._isReactElement) {
    var lastIndent = indent === '' ? '' : indent.replace(/\s\s$/, '');
    var _name = typeof x.type === 'string' ? x.type : x.type.displayName || 'Unknown';
    var keys = Object.keys(x.props);
    var props = '';
    var children = null;
    if (keys.length > 0) {
      seen.push(x);
      var _keys = keys.filter(function (key) {
        return key !== 'children';
      });
      if (_keys.length > 0) {
        var joint = indent === '' ? ' ' : indent;
        props = _keys.map(function (key) {
          return key + '=' + wrapJsxProp(next(x.props[key]));
        });
        props = '' + joint + props.join(joint) + lastIndent;
      }
      if (keys.indexOf('children') !== -1) {
        children = wrapJsxChildren(next(x.props.children));
      }
      seen.pop();
    }
    var ending = children === null ? '/>' : '>' + indent + children + lastIndent + '</' + _name + '>';
    return '<' + _name + props + ending;
  }
  return null;
}

function handleObject(x, next, seen, indent) {
  if (_isobject2['default'](x)) {
    var keys = Object.keys(x);
    if (keys.length === 0) {
      return '{}';
    }
    seen.push(x);
    var joint = indent === '' ? ', ' : ',' + indent;
    var lastIndent = indent === '' ? '' : indent.replace(/\s\s$/, '');
    var body = keys.map(function (key) {
      return key + ': ' + next(x[key]);
    }).join(joint);
    var result = '{' + indent + body + lastIndent + '}';
    seen.pop();
    return result;
  }
  return null;
}

function handleUnknown() {
  return '__UnknownType';
}

var handlers = [handlePrimitive, handleFunction, handleString, handleDate, handleArray, handleJsx, handleObject, handleUnknown];

function stringify(_x, opts) {
  var seen = [];
  function s(x) {
    var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var _ref$indent = _ref.indent;
    var indent = _ref$indent === undefined ? '\n  ' : _ref$indent;
    var _ref$depthLim = _ref.depthLim;
    var depthLim = _ref$depthLim === undefined ? 0 : _ref$depthLim;

    if (seen.indexOf(x) !== -1) {
      return '__Circular';
    }
    var i = 0;
    var result = undefined;
    var next = function next(__x) {
      return s(__x, { indent: indent + '  ', depthLim: depthLim - 1 });
    };
    do {
      result = handlers[i](x, next, seen, depthLim > 0 ? indent : '');
      i++;
    } while (result === null);
    return result;
  }
  return s(_x, opts);
}

module.exports = exports['default'];